"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const dependency_container_1 = require("./dependency-container");
const injectionTokenMetadataKey = "injectionTokens";
function injectable() {
    return function (target) {
        const params = Reflect.getMetadata("design:paramtypes", target) || [];
        const injectionTokens = Reflect.getOwnMetadata(injectionTokenMetadataKey, target) || {};
        Object.keys(injectionTokens).forEach(key => {
            params[+key] = injectionTokens[key];
        });
        return class extends target {
            constructor(...args) {
                let container = dependency_container_1.instance;
                if (args[0] instanceof dependency_container_1.DependencyContainer) {
                    container = args.shift();
                }
                const resolvedArgs = args.slice();
                params.slice(args.length).forEach(param => {
                    resolvedArgs.push(container.resolve(param));
                });
                super(...resolvedArgs);
            }
        };
    };
}
exports.injectable = injectable;
function inject(token) {
    return function (target, _propertyKey, parameterIndex) {
        const injectionTokens = Reflect.getOwnMetadata(injectionTokenMetadataKey, target) || {};
        injectionTokens[parameterIndex] = token;
        Reflect.defineMetadata(injectionTokenMetadataKey, injectionTokens, target);
    };
}
exports.inject = inject;
function registry(providers = [], container) {
    return function (target) {
        providers.forEach(provider => (container || dependency_container_1.instance).register(provider));
        return target;
    };
}
exports.registry = registry;
