"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const providers_1 = require("./providers");
class DependencyContainer {
    constructor(parent) {
        this.parent = parent;
        this._registry = new Map();
    }
    register(provider) {
        if (!providers_1.isClassProvider(provider) && !providers_1.isTokenProvider(provider) && !providers_1.isValueProvider(provider) && !providers_1.isFactoryProvider(provider)) {
            if (!this.isRegistered(provider)) {
                this._registry.set(provider, [provider, undefined]);
            }
        }
        else {
            if (!this.isRegistered(provider.token)) {
                this._registry.set(provider.token, [provider, undefined]);
            }
        }
    }
    resolve(token) {
        const registration = this.getRegistration(token);
        if (!registration) {
            if (typeof (token) === "string") {
                throw `Attempted to resolve unregistered dependency token: ${token}`;
            }
        }
        if (registration) {
            const provider = registration[0];
            const cachedInstance = registration[1];
            if (cachedInstance != undefined) {
                return cachedInstance;
            }
            if (providers_1.isValueProvider(provider)) {
                return registration[1] = provider.useValue;
            }
            else if (providers_1.isTokenProvider(provider)) {
                return registration[1] = this.resolve(provider.useToken);
            }
            else if (providers_1.isClassProvider(provider)) {
                return registration[1] = this.construct(provider.useClass);
            }
            else if (providers_1.isFactoryProvider(provider)) {
                return provider.useFactory(this);
            }
            else {
                return registration[1] = this.construct(provider);
            }
        }
        return this.construct(token);
    }
    isRegistered(token) {
        return this._registry.has(token);
    }
    reset() {
        this._registry.clear();
    }
    createChildContainer() {
        return new DependencyContainer(this);
    }
    getRegistration(token) {
        if (this.isRegistered(token)) {
            return this._registry.get(token);
        }
        if (this.parent) {
            return this.parent.getRegistration(token);
        }
        return null;
    }
    construct(ctor) {
        return this.parent ? new ctor(this) : new ctor();
    }
}
exports.DependencyContainer = DependencyContainer;
exports.instance = new DependencyContainer();
